import tweepy
from django.conf import settings
from django.db import transaction

from twitter_feed.models import Tweet


class ImportTweets:

    def __init__(self):
        self.consumer_key = settings.TWITTER_FEED_CONSUMER_PUBLIC_KEY
        self.consumer_secret = settings.TWITTER_FEED_CONSUMER_SECRET
        self.o_auth_token = settings.TWITTER_FEED_OPEN_AUTH_TOKEN
        self.o_auth_secret = settings.TWITTER_FEED_OPEN_AUTH_SECRET

    def update_tweets(self):
        raw_tweets = self._get_latest_tweets_from_api()
        tweets = [self._tweepy_status_to_tweet(status=status) for status in raw_tweets]
        self._replace_all_tweets(tweets)

    def _get_latest_tweets_from_api(self):
        """
        http://pythonhosted.org/tweepy/html/index.html
        """
        auth = tweepy.OAuthHandler(self.consumer_key, self.consumer_secret)
        auth.set_access_token(self.o_auth_token, self.o_auth_secret)
        api = tweepy.API(auth)

        return api.user_timeline()

    def _tweepy_status_to_tweet(self, status):
        """
        Fields documentation: https://dev.twitter.com/docs/api/1.1/get/statuses/home_timeline
        """
        tweet = Tweet()
        tweet.published_at = status.created_at
        tweet.content = status.text

        return tweet

    @transaction.commit_manually
    def _replace_all_tweets(self, new_tweets):
        try:
            with transaction.commit_manually():
                Tweet.objects.remove_all()

                for tweet in new_tweets:
                    tweet.save()

                transaction.commit()
        except Exception:
            pass


from django.db import models


class TweetManager(models.Manager):

    def get_latest_tweets(self, offset=0, limit=10):
        return self.all().order_by('-published_at')[offset:limit]

    def remove_all(self):
        self.all().delete()


class Tweet(models.Model):
    """
    Cached imported tweet
    """
    content = models.TextField(u"Tweet Content", max_length=20000)
    published_at = models.DateTimeField(u"Published At")
    updated_at = models.DateTimeField(u"Last Update", auto_now=True)
    created_at = models.DateTimeField(u"Date", auto_now_add=True)

    objects = TweetManager()

    class Meta:
        verbose_name = 'Tweet'
        verbose_name_plural = 'Tweets'

    def __unicode__(self):
        return self.content


import os
from setuptools import setup

README = open(os.path.join(os.path.dirname(__file__), 'README.md')).read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-twitter-feed',
    version='0.1',
    packages=['twitter_feed'],
    include_package_data=True,
    license='MIT License',
    description='A simple Django app to show a Twitter feed.',
    long_description=README,
    url='https://github.com/FrancoisConstant/django-twitter-feed',
    author='Francois Constant',
    author_email='francois.constant@gmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
    zip_safe=False,
)